{ File   : GENSVGA.PAS
  Author : Deniz Oezmen, based on an unknown graphics unit
  Created: 1999-05-10
  Changed: 2005-08-26

  Wrapper unit to provide various SVGA graphics modes.
  Warning: Refrain from use. It's quite a mess ...
}
unit GenSVGA;

interface

const
  AutoDetect     = 'Detect';
  GenericSVGA16  = 'SVGA16';
  GenericSVGA256 = 'SVGA256';
  GenericSVGA32K = 'SVGA32K';
  GenericSVGA64K = 'SVGA64K';
  Version = $0B02;
  Build   = $0808;
  Author  = 'Deniz Oezmen, 1999';

procedure SetVideo(XRes, YRes, ColorNum: Longint; var Error: Boolean);
procedure ChooseGraphMode(XRes, YRes, ColorNum: Longint; var Error: Boolean);
procedure WaitForKey;
procedure GraphError(GraphErrorCode: Integer);
procedure VideoInit(PGraphDriverName: string; PGraphMode: Integer);
procedure VideoDeInit;
procedure SelectDriver;

function RealPixelColor(PixColor : Longint): Longint;
function RealDrawColor(Color : Longint): Longint;
function RealFillColor(Color : Longint): Longint;
function RealColor(Color : Longint): Longint;
function WhitePixel: Longint;
function BluePixel: Longint;
function GreenPixel: Longint;
procedure DefaultColors;
function cRGB(R, G, B: Longint) : Longint;

implementation

uses
  TypeChg2,
  Crt,
  Graph;

var
  GraphDriver,
  GraphMode,
  GraphErrorCode,
  MaxXCoord,
  MaxYCoord,
  XRes,
  YRes,
  FontHeight     : Integer;
  MaxColor,
  ColorDepth     : Longint;
  GraphDriverName,
  DriverPath     : string;

procedure GenericSVGA16Driver; external;
{$L SVGA16.OBJ}
procedure GenericSVGA256Driver; external;
{$L SVGA256.OBJ}
procedure GenericSVGA32KDriver; external;
{$L SVGA32K.OBJ}
procedure GenericSVGA64KDriver; external;
{$L SVGA64K.OBJ}

procedure SetVideo(XRes, YRes, ColorNum: Longint; var Error: Boolean);
begin
  ChooseGraphMode(XRes, YRes, ColorNum, Error)
end;

procedure ChooseGraphMode(XRes, YRes, ColorNum: Longint; var Error: Boolean);
begin
  Error := False;
  if ColorNum = 16 then
  begin
    case XRes of
      0320: case YRes of
              0200: VideoInit(GenericSVGA16, 0)
            else
              Error := True
            end;
      0640: case YRes of
              0200: VideoInit(GenericSVGA16, 1);
              0350: VideoInit(GenericSVGA16, 2);
              0480: VideoInit(GenericSVGA16, 3)
            else
              Error := True
            end;
      0800: case YRes of
              0600: VideoInit(GenericSVGA16, 4)
            else
              Error := True
            end;
      1024: case YRes of
              0768: VideoInit(GenericSVGA16, 5)
            else
              Error := True
            end;
      1280: case YRes of
              1024: VideoInit(GenericSVGA16, 6)
            else
              Error := True
            end
    else
      Error := True
    end
  end
  else if ColorNum = 256 then
  begin
    case XRes of
      0320: case YRes of
              0200: VideoInit(GenericSVGA256, 0)
            else
              Error := True
            end;
      0640: case YRes of
              0400: VideoInit(GenericSVGA256, 1);
              0480: VideoInit(GenericSVGA256, 2);
              0350: VideoInit(GenericSVGA256, 5)
            else
              Error := True
            end;
      0800: case YRes of
              0600: VideoInit(GenericSVGA256, 3)
            else
              Error := True
            end;
      1024: case YRes of
              0768: VideoInit(GenericSVGA256, 4)
            else
              Error := True
            end;
      1280: case YRes of
              1024: VideoInit(GenericSVGA256, 6)
            else
              Error := True
            end
    else
      Error := True
    end
  end
  else if ColorNum = 32768 then
  begin
    case XRes of
      0320: case YRes of
              0200: VideoInit(GenericSVGA32K, 0)
            else
              Error := True
            end;
      0640: case YRes of
              0350: VideoInit(GenericSVGA32K, 1);
              0400: VideoInit(GenericSVGA32K, 2);
              0480: VideoInit(GenericSVGA32K, 3)
            else
              Error := True
            end;
      0800: case YRes of
              0600: VideoInit(GenericSVGA32K, 4)
            else
              Error := True
            end;
      1024: case YRes of
              0768: VideoInit(GenericSVGA32K, 5)
            else
              Error := True
            end;
      1280: case YRes of
              1024: VideoInit(GenericSVGA32K, 6)
            else
              Error := True
            end
    else
      Error := True
    end
  end
  else if ColorNum = 65536 then
  begin
    case XRes of
      0320: case YRes of
              0200: VideoInit(GenericSVGA64K, 0)
            else
              Error := True
            end;
      0640: case YRes of
              0350: VideoInit(GenericSVGA64K, 1);
              0400: VideoInit(GenericSVGA64K, 2);
              0480: VideoInit(GenericSVGA64K, 3)
            else
              Error := True
            end;
      0800: case YRes of
              0600: VideoInit(GenericSVGA64K, 4)
            else
              Error := True
            end;
      1024: case YRes of
              0768: VideoInit(GenericSVGA64K, 5)
            else
              Error := True
            end;
      1280: case YRes of
              1024: VideoInit(GenericSVGA64K, 6)
            else
              Error := True
            end
    else
      Error := True
    end
  end
  else
    Error := True
end;

procedure SelectDriver;
begin
  if GraphDriverName = GenericSVGA16 then
  begin
    InstallUserDriver('SVGA16', nil);
    GraphDriver := RegisterBGIDriver(@GenericSVGA16Driver)
  end
  else if GraphDriverName = GenericSVGA256 then
  begin
    InstallUserDriver('SVGA256', nil);
    GraphDriver := RegisterBGIDriver(@GenericSVGA256Driver)
  end
  else if GraphDriverName = GenericSVGA32K then
  begin
    InstallUserDriver('SVGA32K', nil);
    GraphDriver := RegisterBGIDriver(@GenericSVGA32KDriver)
  end
  else if GraphDriverName = GenericSVGA64K then
  begin
    InstallUserDriver('SVGA64K', nil);
    GraphDriver := RegisterBGIDriver(@GenericSVGA64KDriver)
  end
end;

procedure VideoInit(PGraphDriverName: string; PGraphMode: Integer);
begin
  GraphDriverName := PGraphDriverName;
  GraphMode := PGraphMode;
  DriverPath := '';
  DirectVideo := False;
  SelectDriver;

  GraphDriver := 16;

  InitGraph(GraphDriver, GraphMode, DriverPath);

  GraphErrorCode := GraphResult;
  if GraphErrorCode and $80 = $80 then
    GraphErrorCode := GraphErrorCode or $FF00;

  if GraphErrorCode <> grOK then
    GraphError(GraphErrorCode);

  (* Set Defaults *)
  DefaultColors;
  {SetTextStyle(SansSerifFont, HorizDir, 1);}
  MaxXCoord := GetMaxX;
  MaxYCoord := GetMaxY;
  MaxColor := GetMaxColor;
  XRes := MaxXCoord + 1;
  YRes := MaxYCoord + 1;
  ColorDepth := MaxColor + 1;
  FontHeight := TextHeight('I')
  (* end Set Defaults *)
end;

procedure GraphError(GraphErrorCode: Integer);
begin
  CloseGraph;
  RestoreCrtMode;
  WriteLn('Critical graphics error: ', GraphErrorMsg(GraphErrorCode));
  Halt
end;

procedure VideoDeInit;
begin
  ClearDevice;
  CloseGraph;
  RestoreCrtMode
end;

procedure WaitForKey;
begin
  while ReadKey = '' do
end;

(* Copied stuff *)

function RealPixelColor(PixColor: Longint): Longint;
var
  CurC: Integer;
begin
  RealPixelColor := PixColor;
end;

function RealDrawColor(Color: Longint): Longint;
var
  MaxC: Longint;
begin
  MaxC := GetMaxColor;

  if (MaxC = 65535) then
    SetRgbPalette(1024, (Color shr 11) and 31, (Color shr 5) and 63,
      Color and 31)
  else if (MaxC = 32767) then
    SetRgbPalette(1024, (Color shr 10) and 31, (Color shr 5) and 31,
      Color and 31)
  else if (MaxC = 16777) then
  begin
    SetRgbPalette(1024, (Color shr 16) and 255, (Color shr 8) and 255,
      Color and 255)
  end;
  RealDrawColor := Color
end;

function RealFillColor(Color: Longint): Longint;
var
  MaxC: Longint;
begin
  MaxC := GetMaxColor;

  if (MaxC = 65535) then
    SetRgbPalette(1025, (Color shr 11) and 31, (Color shr 5) and 63,
      Color and 31)
  else if (MaxC = 32767) then
    SetRgbPalette(1025, (Color shr 10) and 31, (Color shr 5) and 31,
      Color and 31)
  else if (MaxC = 16777) then
  begin
    SetRgbPalette(1025, (Color shr 16) and 255, (Color shr 8) and 255,
      Color and 255);
    Color := 0
  end;
  RealFillColor := Color
end;

function RealColor(Color: Longint): Longint;
var
  MaxC: Longint;
begin
  MaxC := GetMaxColor;

  if (MaxC = 65535) then
    SetRgbPalette(1026, (Color shr 11) and 31, (Color shr 5) and 63,
      Color and 31)
  else if (MaxC = 32767) then
    SetRgbPalette(1026, (Color shr 10) and 31, (Color shr 5) and 31,
      Color and 31)
  else if (MaxC = 16777) then
  begin
    SetRgbPalette(1026, (Color shr 16) and 255, (Color shr 8) and 255,
      Color and 255);
    Color := 0
  end;
  RealColor := Color
end;

function WhitePixel: Longint;
var
  Clr: Longint;
begin
  Clr := GetMaxColor;

  if (Clr = 65535) then
    Clr := $ffff
  else if (Clr = 32767) then
    Clr := $7fff
  else if (Clr = 16777) then
    Clr := $ffffff
  else
    Clr := 15;
  WhitePixel := Clr
end;

function BluePixel: Longint;
var
  Clr: Longint;
begin
  Clr := GetMaxColor;

  if (Clr = 65535) then
    Clr := $1f
  else if (Clr = 32767) then
    Clr := $1f
  else if (Clr = 16777) then
    Clr := $ff
  else
    Clr := 1;
  BluePixel := Clr
end;

function GreenPixel: Longint;
var
  Clr: Longint;
begin
  Clr := GetMaxColor;

  if (Clr = 65535) then
    Clr := 63 shl 5
  else if (Clr = 32767) then
    Clr := 31 shl 5
  else if (Clr = 16777) then
    Clr := $ff00
  else
    Clr := 2;
  GreenPixel := Clr
end;

procedure DefaultColors;
{ Select the maximum color in the Palette for the drawing color }
begin
  SetColor(RealDrawColor(WhitePixel))
end; { DefaultColors }

function cRGB(R, G, B: Longint): Longint;
begin
  if (GetMaxColor = 32767) then
    cRGB := (((R shr 1) and 31) shl 10) + (((G shr 1) and 31) shl 5) +
            ((B shr 1) and 31)
  else if (GetMaxColor = 65535) then
    cRGB := (((R shr 1) and 31) shl 11) or (((G {shr 1}) and 63) shl 5) or
            ((B shr 1) and 31)
  else
    cRGB := (R shl 18) + (G shl 10) + (B shl 2)
end;

(* end copied stuff *)


begin
end.